/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.servicemix.ldap.marshaler;

import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.naming.NamingEnumeration;

/**
 * This is the common marshaler interface for all LDAP marshalers
 * 
 * @author jbonofre
 */
public interface LdapMarshalerSupport {
    
	/**
	 * Converts a LDAP JNDI NamingEnumeration result into a normalized message send
	 * in the NMR
	 *  
	 * @param message the <code>NormalizedMessage</code> to send into the NMR
	 * @param namingEnumeration the <code>NamingEnumeration</code> LDAP JNDI search result
	 * @throws MessagingException in case of error during normalized message manipulation
	 */
    void marshal(NormalizedMessage message, NamingEnumeration namingEnumeration) throws MessagingException;

}
