/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.servicemix.nmr.api.security;

import java.security.Principal;


/**
 * This principal represents a group against which authorizations are
 * usually checked.
 */
public class GroupPrincipal implements Principal {

    public static final GroupPrincipal ANY = new GroupPrincipal("*");

    private final String name;
    private transient int hash;
    private String containerRolePrincipalClassName = "org.apache.karaf.jaas.boot.principal.RolePrincipal";

    public GroupPrincipal(String name) {
        if (name == null) {
            throw new IllegalArgumentException("name cannot be null");
        }
        this.name = name;
    }

    public GroupPrincipal(String name, String containerRolePrincipalClassName) {
        this(name);
        this.containerRolePrincipalClassName = containerRolePrincipalClassName;
    }
    
    public String getName() {
        return name;
    }

    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null 
            || (!o.getClass().getName().equals(containerRolePrincipalClassName) && getClass() != o.getClass())) {
        	return false;
        }
        final Principal that = (Principal) o;
        if (!name.equals(that.getName())) {
            return false;
        }
        return true;
    }

    public int hashCode() {
        if (hash == 0) {
            hash = name.hashCode();
        }
        return hash;
    }

    public String toString() {
        return name;
    }
}
